UNIT Unit1;

INTERFACE

USES
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  Spin, ExtDlgs;

TYPE
  TForm1 = CLASS(TForm)
    Panel1: TPanel;
    Edit1: TEdit;
    Button1: TButton;
    SpinEdit1: TSpinEdit;
    Button3: TButton;
    Scales: TRadioGroup;
    Button2: TButton;
    Panel2: TPanel;
    Image1: TImage;
    Use_scaling: TCheckBox;
    opd: TOpenPictureDialog;
    Label1: TLabel;
    Panel3: TPanel;
    PROCEDURE Button1Click(Sender: TObject);
    PROCEDURE Button2Click(Sender: TObject);
    PROCEDURE FormPaint(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  END;

VAR
  Form1: TForm1;
  TimingSeconds: real;
IMPLEMENTATION
USES jpeg,
  math;

{$R *.dfm}
{$R XPTheme.res}

PROCEDURE ScaleJPEG(Scale_to: integer; JP: tJPEGImage);

  PROCEDURE DrawJPEG(JPG: tJPEGImage); // resize jpeg to scaled size
  VAR
    Bmp: tBitmap;
  BEGIN
    Bmp := tBitmap.Create;
    TRY
      Bmp.Width := JPG.Width;
      Bmp.Height := JPG.Height;
      Bmp.PixelFormat := pf24bit;
      Bmp.Canvas.draw(0, 0, JPG);
      Bmp.freeimage;
      JPG.Assign(Bmp);
    FINALLY
      Bmp.Free;
    END;
  END;

VAR
  X, Y: tJPEGScale;
  Strm: tMemoryStream;
BEGIN

  CASE Scale_To OF // set JPEG scaling vars
    0..3: X := tJPEGScale(Scale_To);
    4..6: BEGIN
        X := jsEighth;
        Y := tJpegScale(Scale_to - 3); END;
  ELSE
    ShowMessage('Size must be between 0 and 6');
    EXIT;
  END;

  JP.Scale := X;

  DrawJPEG(JP); // draw scaled jpeg to bitmap and reassign it back to the jpeg

  IF Scale_To > 3 THEN BEGIN // repeat if scale is smaller than jsEighth

    //JP.JPEGNeeded;
    //This works for 16th and 32th scaling but doesn't work for 64th
    //and I can't see why not!  Streaming in and out works but no doubt
    //has an overhead?

    Strm := tMemoryStream.Create;
    TRY
      JP.SaveToStream(Strm);
      Strm.Seek(0, 0);
      JP.LoadFromStream(Strm);
    FINALLY
      Strm.Free;
    END;

    jp.Scale := y;
    DrawJPEG(JP);
  END;
END;

//StretchBlt is quick and in the main gives good output as long as the
//difference between src and dest is not too great.

PROCEDURE StretchImage(Src, Dst: TBitmap);
BEGIN
  SetStretchBltMode(Dst.Canvas.Handle, HALFTONE);
  StretchBlt(Dst.Canvas.Handle, 0, 0, Dst.Width, Dst.Height,
    Src.Canvas.Handle, 0, 0, Src.Width, Src.Height, SRCCOPY);
END;

PROCEDURE ResizeJPEG(Size_to: integer; JP: tJPEGImage; UseScale: Boolean = TRUE);
VAR
  A, B: tBitmap;
  M, W, H: integer;
BEGIN
  W := JP.Width;
  H := JP.Height;

  IF UseScale THEN BEGIN // work out the nearest scaled size to final size
    CASE (max(W, H) DIV Size_To) OF
      0..1: M := 0;
      2..3: M := 1;
      4..7: M := 2;
      8..15: M := 3;
      16..31: M := 4;
      32..63: M := 5;
    ELSE
      M := 6;
    END;

    ScaleJPEG(M, JP); //resize the jpeg to near final size
  END;

  A := TBitmap.Create;

  TRY
    A.PixelFormat := pf24bit;

    A.Assign(JP); // assign jpeg to the src bitmap

    B := tBitmap.Create; // create and size the dest bitmap

    TRY
      B.PixelFormat := pf24bit;

      IF W > H THEN BEGIN
        B.Width := Size_To;
        B.Height := muldiv(Size_To, H, W);
      END
      ELSE BEGIN
        B.Height := Size_To;
        B.Width := MulDiv(Size_To, W, H);
      END;

      StretchImage(A, B); // resize using StretchBlt

      JP.Assign(B); // reassign dest bitmap to original jpeg
    FINALLY
      B.Free;
    END;

  FINALLY
    A.Free;
  END;

END;

PROCEDURE TForm1.Button1Click(Sender: TObject);
VAR
  JP: tJPEGImage;
  Freq, StartCount, StopCount: Int64;
BEGIN
  JP := tJPEGImage.Create;
  TRY
    jp.LoadFromFile(edit1.text);
    QueryPerformanceFrequency(Freq);
    QueryPerformanceCounter(StartCount);
    ResizeJPEG(spinedit1.value, JP, Use_Scaling.Checked);
    QueryPerformanceCounter(StopCount);
    TimingSeconds := (StopCount - StartCount) / Freq;
  FINALLY
    Image1.Picture.Assign(JP);
    JP.Free;
  END;
  Panel3.Caption := Format('Size: %d x %d px (%g sec)', [Image1.Picture.Width, Image1.Picture.Height, TimingSeconds]);
END;

PROCEDURE TForm1.Button2Click(Sender: TObject);
VAR
  JP: tJPEGImage;
  Freq, StartCount, StopCount: Int64;
BEGIN
  JP := tJPEGImage.Create;
  TRY
    jp.LoadFromFile(edit1.text);
    QueryPerformanceFrequency(Freq);
    QueryPerformanceCounter(StartCount);
    ScaleJPEG(Scales.ItemIndex, JP);
    QueryPerformanceCounter(StopCount);
    TimingSeconds := (StopCount - StartCount) / Freq;
  FINALLY
    Image1.Picture.Assign(JP);
    JP.Free;
  END;
  panel3.Caption := Format('Size: %d x %d px (%g sec)', [Image1.Picture.Width, Image1.Picture.Height, TimingSeconds]);

END;

PROCEDURE TForm1.FormPaint(Sender: TObject);
BEGIN
  ControlStyle := ControlStyle + [csOpaque];
END;

procedure TForm1.Button3Click(Sender: TObject);
begin
opd.Filter:=graphicfilter(tjpegimage);
if opd.Execute then
edit1.Text:=opd.FileName;
end;

procedure TForm1.Edit1Change(Sender: TObject);
begin
Button1.Enabled:=FileExists(Edit1.Text);
Button2.Enabled:=Button1.Enabled;
end;

END.

